/*
 * Set of functions dedicated to transformations of normals.
 *
 * Dependencies:
 *     - Textures.glsl (if using NORMAL_MAPPING)
 */

/**
 * Contains the data used to shade a surface, i.e.,
 *     - Normal
 *     - Tangent (Optional)
 *     - Bitangent (Optional)
 */
struct SurfaceData {
    normal: vec3<f32>,
#ifdef USE_TANGENT
    tangent: vec3<f32>,
    bitangent: vec3<f32>,
#endif
}

/**
 * Create surface data from raw shader inputs
 *
 * **Note**: This method automatically performs normal flipping
 * for double-sided rendering.
 *
 * @param fragNormal The un-normalized surface normal
 * @returns A struct containing surface data
 */
fn computeSurfaceDataNormal(fragNormal: vec3<f32>) -> SurfaceData {
    var surface: SurfaceData;
    let frontFacing: f32 = 1.0;
    surface.normal = normalize(fragNormal*frontFacing);
    return surface;
}

#ifdef USE_TANGENT

/**
 * Create surface data from raw shader inputs
 *
 * **Note**: This method automatically performs normal flipping
 * for double-sided rendering.
 *
 * @param fragNormal The un-normalized surface normal
 * @param fragTangent The un-normalized surface tangent
 * @returns A struct containing surface data
 */
fn computeSurfaceData(fragNormal: vec3<f32>, fragTangent: vec4<f32>) -> SurfaceData {
    let frontFacing: f32 = 1.0;
    let flip: f32 = frontFacing;
    var surface: SurfaceData;
    surface.normal = normalize(fragNormal*flip);
    surface.tangent = normalize(fragTangent.xyz*flip);
    surface.bitangent = normalize(cross(surface.normal, surface.tangent)*flip*fragTangent.w);
    return surface;
}

#ifdef NORMAL_MAPPING

/**
 * Perturb the given normal using the Normal Mapping technique
 *
 * @param normal The normal to the surface
 * @param tangent The tangent to the surface
 * @param bitangent The bitangent to the surface
 * @param normalRGB Normal RGB texture value
 * @returns The perturbed and normalized normal
 */
fn normalMapping(normal: vec3<f32>, tangent: vec3<f32>,
    bitangent: vec3<f32>, normalRGB: vec3<f32>) -> vec3<f32> {
    let tbn: mat3x3<f32> = mat3x3<f32>(tangent, bitangent, normal);
    return tbn*(normalize(normalRGB*2.0 - vec3<f32>(1.0)));
}

#endif

#endif
